/*
 * lab1.c
 */

#include "lcd.h"

#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <avr/interrupt.h>

volatile uint16_t ADC_value;
int nr_read;
double final_value;
double calibrated_value;

float sensor_volt;
float RS_gas;
float R0;
int R2 = 2000;

#define ADC_INIT_CHANNEL 0

void ADC_init(void) {
	ADMUX = 0;
	ADCSRA = 0;
	nr_read = 0;
	final_value = 0;

	ADMUX = (1 << REFS0) | ADC_INIT_CHANNEL;

	// 128 prescale for 8Mhz
	ADCSRA |= (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0);   
	ADCSRA |= (1 << ADEN);  // bit enable si activeaza conv
	ADCSRA |= (1 << ADIE);  // enable interrupts	
}

void timer1_start(void) {
	// genereaza o intrerupere care sa poneasca
	// adc-ul la fiecare 100ms
    TCCR1B = (5 << CS10) | (1 << WGM12);
    TIMSK1 |= (1 << OCIE1A);
    OCR1A = 1561;
}

ISR(ADC_vect)
{
    // Intreruperea este triggered cand o conversie este finalizata. */

    ADC_value = ADC;
	nr_read++;
   
}

ISR(TIMER1_COMPA_vect) {
	// porneste o conversie ADC
	ADCSRA |= (1 << ADSC);
}

double ADC_get(uint8_t channel)
{
	double final_value_copy;
	final_value += ((float) ADC_value * 0.15) / 400;
	if (nr_read == 50) {
		final_value_copy = final_value;
		nr_read = 0;
		final_value = 0;
		return final_value_copy;
	}

	return 0;

}


int main()
{
	char row1[17], row2[17];
	int calibrated = 0;
	int stop = 0;

	LCD_init();
	ADC_init();
	timer1_start();
	sei();  // enable interrupts
	DDRD |= (1 << PD5);
	sprintf(row1, "Calibrating...");
	LCD_printAt(0, "                ");
	LCD_printAt(0, row1);

	calibrated_value = ADC_get(0);
	while (calibrated_value == 0) {
		
		calibrated_value = ADC_get(0);
		LCD_printAt(0, "                ");
		LCD_printAt(0, row1);
	}
	calibrated_value /= 50;
	sprintf(row1, "Calibrated");
	sprintf(row2, "Suflati 5 sec");
	LCD_printAt(0, "                ");
	LCD_printAt(0, row1);
	LCD_printAt(0x40, "                ");
	LCD_printAt(0x40, row2);

	double alc_level;
    while (1) {
		alc_level = ADC_get(0);
		while (alc_level == 0) {
			alc_level = ADC_get(0);
			if (stop == 1) {
				PIND = (1 << PD5);
			}
		}
		alc_level /= 50;
		if (calibrated == 0) {
			calibrated_value = alc_level;
			calibrated = 1;
		}
		alc_level -= calibrated_value;
		if (alc_level < 0) {
			alc_level = 0;
		}
		char snum[5];
		itoa(alc_level, snum, 10);
		if (alc_level < 39) {
			sprintf(row1, "Nu a/i baut nimic");
			sprintf(row2, "0.0%s g/l", snum);
			stop = 0;
		} else if (alc_level < 69) {
			sprintf(row1, "Ati baut putin");
			sprintf(row2, "0.0%s g/l", snum);
			stop = 0;
		} else if (alc_level < 110) {
			sprintf(row1, "Nu mai beti");
			if (alc_level <= 99) {
				sprintf(row2, "0.0%s g/l", snum);
			} else {
				sprintf(row2, "0.%s g/l", snum);
			}
			stop = 0;
		} else {
			sprintf(row1, "STOP");
			sprintf(row2, "0.%s g/l", snum);
			PIND = (1 << PD5);
			stop = 1;
		}
		LCD_printAt(0, "                ");
		LCD_printAt(0, row1);
		LCD_printAt(0x40, "                ");
		LCD_printAt(0x40, row2);
    }
       
    return 0;
}

